@@ -32,22 +32,18 @@ use File::Basename;
 use File::Spec;
 use File::pfopen 0.02;
 use File::Temp;
-use Error::Simple;
-use Database::Abstraction::Error;
+# use Error::Simple;	# A nice idea to use this but it doesn't play well with "use lib"
 use Carp;
 
-our $directory;
-our $logger;
-our $cache;
-our $cache_duration;
+our %defaults;
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 =head1 SYNOPSIS
 
@@ -97,10 +93,11 @@ See the documentation for new() to see what variables can be set
 sub init {
 	my %args = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
 
-	$directory ||= $args{'directory'};
-	$logger ||= $args{'logger'};
-	$cache ||= $args{'cache'};
-	$cache_duration ||= $args{'cache_duration'};
+	# $defaults->{'directory'} ||= $args{'directory'};
+	# $defaults->{'logger'} ||= $args{'logger'};
+	# $defaults->{'cache'} ||= $args{'cache'};
+	# $defaults->{'cache_duration'} ||= $args{'cache_duration'};
+	%defaults = %args;
 }
 
 =head2 new
@@ -142,17 +139,20 @@ sub new {
 		return bless { %{$class}, %args }, ref($class);
 	}
 
-	croak("$class: where are the files?") unless($directory || $args{'directory'});
+	croak("$class: where are the files?") unless($args{'directory'} || $defaults{'directory'});
+
+	croak("$class: ", $args{'directory'} || $defaults{'directory'}, ' is not a directory') unless(-d ($args{'directory'} || $defaults{'directory'}));
 	# init(\%args);
 
-	return bless {
-		logger => $args{'logger'} || $logger,
-		directory => $args{'directory'} || $directory,	# The directory containing the tables in XML, SQLite or CSV format
-		cache => $args{'cache'} || $cache,
-		cache_duration => $args{'cache_duration'} || $cache_duration || '1 hour',
-		table => $args{'table'},	# The name of the file containing the table, defaults to the class name
-		no_entry => $args{'no_entry'} || 0,
-	}, $class;
+	# return bless {
+		# logger => $args{'logger'} || $logger,
+		# directory => $args{'directory'} || $directory,	# The directory containing the tables in XML, SQLite or CSV format
+		# cache => $args{'cache'} || $cache,
+		# cache_duration => $args{'cache_duration'} || $cache_duration || '1 hour',
+		# table => $args{'table'},	# The name of the file containing the table, defaults to the class name
+		# no_entry => $args{'no_entry'} || 0,
+	# }, $class;
+	return bless { %args, %defaults, no_entry => 0 }, $class;
 }
 
 =head2	set_logger
@@ -204,7 +204,7 @@ sub _open {
 	# Read in the database
 	my $dbh;
 
-	my $dir = $self->{'directory'} || $directory;
+	my $dir = $self->{'directory'} || $defaults{'directory'};
 	my $slurp_file = File::Spec->catfile($dir, "$table.sql");
 	if($self->{'logger'}) {
 		$self->{'logger'}->debug("_open: try to open $slurp_file");
@@ -355,7 +355,8 @@ sub _open {
 				}
 				$dbh->func($table, 'XML', $slurp_file, 'xmlsimple_import');
 			} else {
-				throw Database::Abstraction::Error(-file => "$dir/$table");
+				# throw Error(-file => "$dir/$table");
+				croak("Can't file a $table database in $dir");
 			}
 			$self->{'type'} = 'XML';
 		}
@@ -425,12 +426,15 @@ sub selectall_hash {
 			if($self->{'logger'}) {
 				$self->{'logger'}->fatal("selectall_hash $query: argument is not a string");
 			}
-			throw Error::Simple("$query: argument is not a string: " . ref($arg));
+			# throw Error::Simple("$query: argument is not a string: " . ref($arg));
+			croak("$query: argument is not a string: ", ref($arg));
 		}
 		if(!defined($arg)) {
 			my @call_details = caller(0);
-			throw Error::Simple("$query: value for $c1 is not defined in call from " .
-				$call_details[2] . ' of ' . $call_details[1]);
+			# throw Error::Simple("$query: value for $c1 is not defined in call from " .
+				# $call_details[2] . ' of ' . $call_details[1]);
+			croak("$query: value for $c1 is not defined in call from ",
+				$call_details[2], ' of ', $call_details[1]);
 		}
 		if($done_where) {
 			if($arg =~ /\@/) {
@@ -487,7 +491,8 @@ sub selectall_hash {
 
 	if(my $sth = $self->{$table}->prepare($query)) {
 		$sth->execute(@query_args) ||
-			throw Error::Simple("$query: @query_args");
+			# throw Error::Simple("$query: @query_args");
+			croak("$query: @query_args");
 
 		my @rc;
 		while(my $href = $sth->fetchrow_hashref()) {
@@ -504,7 +509,8 @@ sub selectall_hash {
 	if($self->{'logger'}) {
 		$self->{'logger'}->warn("selectall_hash failure on $query: @query_args");
 	}
-	throw Error::Simple("$query: @query_args");
+	# throw Error::Simple("$query: @query_args");
+	croak("$query: @query_args");
 }
 
 =head2	fetchrow_hashref
@@ -579,7 +585,8 @@ sub fetchrow_hashref {
 		}
 	}
 	my $sth = $self->{$table}->prepare($query) or die $self->{$table}->errstr();
-	$sth->execute(@query_args) || throw Error::Simple("$query: @query_args");
+	# $sth->execute(@query_args) || throw Error::Simple("$query: @query_args");
+	$sth->execute(@query_args) || croak("$query: @query_args");
 	if($c) {
 		my $rc = $sth->fetchrow_hashref();
 		$c->set($key, $rc, $self->{'cache_duration'});
@@ -620,7 +627,8 @@ sub execute {
 		$self->{'logger'}->debug("execute $query");
 	}
 	my $sth = $self->{$table}->prepare($query);
-	$sth->execute() || throw Error::Simple($query);
+	# $sth->execute() || throw Error::Simple($query);
+	$sth->execute() || croak($query);
 	my @rc;
 	while(my $href = $sth->fetchrow_hashref()) {
 		return $href if(!wantarray);
@@ -749,8 +757,10 @@ sub AUTOLOAD {
 			$self->{'logger'}->debug("AUTOLOAD $query");
 		}
 	}
-	my $sth = $self->{$table}->prepare($query) || throw Error::Simple($query);
-	$sth->execute(@args) || throw Error::Simple($query);
+	# my $sth = $self->{$table}->prepare($query) || throw Error::Simple($query);
+	my $sth = $self->{$table}->prepare($query) || croak($query);
+	# $sth->execute(@args) || throw Error::Simple($query);
+	$sth->execute(@args) || croak($query);
 
 	if(wantarray) {
 		return map { $_->[0] } @{$sth->fetchall_arrayref()};
@@ -776,6 +786,11 @@ sub DESTROY {
 
 Nigel Horne, C<< <njh at bandsman.co.uk> >>
 
+=head1 BUGS
+
+The default delimiter for CSV files is set to '!', not ',' for historical reasons.
+I really ought to fix that.
+
 =head1 LICENSE AND COPYRIGHT
 
 Copyright 2015-2024 Nigel Horne.
